@@ -27,9 +27,25 @@ from httmock import HTTMock  # noqa
 from httmock import response  # noqa
 from httmock import urlmatch  # noqa
 
+import gitlab
 from gitlab import *  # noqa
 
 
+class TestSanitize(unittest.TestCase):
+    def test_do_nothing(self):
+        self.assertEqual(1, gitlab._sanitize(1))
+        self.assertEqual(1.5, gitlab._sanitize(1.5))
+        self.assertEqual("foo", gitlab._sanitize("foo"))
+
+    def test_slash(self):
+        self.assertEqual("foo%2Fbar", gitlab._sanitize("foo/bar"))
+
+    def test_dict(self):
+        source = {"url": "foo/bar", "id": 1}
+        expected = {"url": "foo%2Fbar", "id": 1}
+        self.assertEqual(expected, gitlab._sanitize(source))
+
+
 class TestGitlabRawMethods(unittest.TestCase):
     def setUp(self):
         self.gl = Gitlab("http://localhost", private_token="private_token",
@@ -322,7 +338,7 @@ class TestGitlabMethods(unittest.TestCase):
             self.assertRaises(GitlabGetError, self.gl.get,
                               Project, 1)
 
-    def test_delete(self):
+    def test_delete_from_object(self):
         @urlmatch(scheme="http", netloc="localhost", path="/api/v3/groups/1",
                   method="delete")
         def resp_delete_group(url, request):
@@ -335,6 +351,24 @@ class TestGitlabMethods(unittest.TestCase):
             data = self.gl.delete(obj)
             self.assertIs(data, True)
 
+    def test_delete_from_invalid_class(self):
+        class InvalidClass(object):
+            pass
+
+        self.assertRaises(GitlabError, self.gl.delete, InvalidClass, 1)
+
+    def test_delete_from_class(self):
+        @urlmatch(scheme="http", netloc="localhost", path="/api/v3/groups/1",
+                  method="delete")
+        def resp_delete_group(url, request):
+            headers = {'content-type': 'application/json'}
+            content = ''.encode("utf-8")
+            return response(200, content, headers, None, 5, request)
+
+        with HTTMock(resp_delete_group):
+            data = self.gl.delete(Group, 1)
+            self.assertIs(data, True)
+
     def test_delete_unknown_path(self):
         obj = Project(self.gl, data={"name": "testname", "id": 1})
         obj._from_api = True
